/* ************************************************************************* */
/*                                                                           */
/* Source: Huffplet.java                        |\      _,,,----,            */
/*                                              /,`.-'`'    -.  ;-;;,_       */
/* Author: Oliver Schmid                       |,4-  ) )-,_. ,\ (  `'-'      */
/* Date:   May 2002                           '---''(_/--'  `-'\_)           */
/*                                                                           */
/* Description: When using Huffman-Trees for coding the letters are listed   */
/*         with their frequencies to become the leaves of a tree being con-  */
/*         structed bottom-up. The two parentless nodes with the lowest fre- */
/*         quency become the children of a new created node being assigned   */
/*         the sum of its children's frequencies. This step is repeated un-  */
/*         til the root is the only remaing parentless node. For each parent */
/*         node the left path is assigned a 0 and the right path a 1.        */
/*            When encoding, each letter searches its path through the tree  */
/*         receiving the bits read top-down as its code.                     */
/*            When decoding, the bit string searches its path to a leaf top- */
/*         down receiving its content as decoded text.                       */
/*                                                                           */
/* ************************************************************************* */



import java.awt.*;
import javax.swing.*;



class Huffplet extends Lossless implements Runnable {



    /* ******************   Graphical output methods   ****************** */



    public void makeLeaf(Graphics graph, FontMetrics fontMet, Color c,
                         String s1, String s2, int x, int y) {
        graph.setColor(c);
        graph.fillOval(x - 15, y - 15, 30, 30);
        graph.setColor(Color.black);
        graph.drawOval(x - 15, y - 15, 30, 30);
        graph.setColor(Color.blue);
        graph.setFont(new Font("Serif", Font.PLAIN, 24));
        graph.drawString(s1, x - fontMet.stringWidth(s1) / 2,
                         y + 9);
        graph.setFont(new Font("Serif", Font.PLAIN, 12));
        graph.setColor(Color.black);
        graph.drawString(s2, x + 15, y + 15);
    }

    public void makeNode(Graphics graph, Color c, String s, int x, int y) {
        graph.setColor(c);
        graph.fillOval(x - 8, y - 8, 15, 15);
        graph.setColor(Color.black);
        graph.drawOval(x - 8, y - 8, 15, 15);
        graph.setFont(new Font("Serif", Font.PLAIN, 12));
        graph.drawString(s, x + 8, y + 8);
    }

    private String input;

    // Constructor
    public Huffplet(String input) {
	this.input = input;
    }
		


    public void run() {



	/* ******************   Applet graphics   ****************** */



	// Create JFrame
//  	JFrame myFrame = new JFrame("Huffman-Code");
//  	myFrame.getContentPane().add(this);
//  	myFrame.setSize(700, 500);
//  	myFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//  	myFrame.setVisible(true);

	Color myBackColor = new Color(207, 207, 239);
//	myFrame.setBackground(myBackColor);

	Container myFrame = this;

	// Create Graphics2D with bufferedImage
	Graphics2D graf = this.buffI.createGraphics();
	graf.setRenderingHint(RenderingHints.KEY_RENDERING,
			   RenderingHints.VALUE_RENDER_QUALITY);
	graf.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
			   RenderingHints.VALUE_ANTIALIAS_ON);
	graf.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
			   RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
	graf.setColor(myBackColor);
	graf.fillRect(0, 0, 700, 500);
	graf.setColor(Color.black);
	graf.setFont(new Font("Serif", Font.PLAIN, 12));

	Font f = new Font("Serif", Font.PLAIN, 12);
	FontMetrics fMet; fMet = getFontMetrics(f);

	Font inCF = new Font("Serif", Font.PLAIN, 24);
	FontMetrics inCMet; inCMet = getFontMetrics(inCF);

	// Set times to wait (in ms)
	int longW = 1000, medW = 250, shrtW = 50;



	/* ****************   Datastructures Huffman-Tree   **************** */



	// Create alphabet
	String alphabet = "";
	for (int i = 0; i < input.length(); i++) {
	    if (alphabet.indexOf(input.charAt(i)) < 0) {
		alphabet += String.valueOf(input.charAt(i));
	    }
	}

	// Create table to count the letters' frequencies
	int[] frequency; frequency = new int[alphabet.length()];
	for (int i = 0; i < frequency.length; i++) {
	    frequency[i] = 0;
	}

	// Count the letters' frequencies
	for (int i = 0; i < input.length(); i++) {
     	    frequency[alphabet.indexOf(input.charAt(i))]++;
	}

	// Create Huffman-Tree
	int nodeNumber = alphabet.length();
	int[] cumulFrequency; cumulFrequency = new int[2 * nodeNumber - 1];
	String[] character; character = new String[nodeNumber];
	int[] dad; dad = new int[2 * nodeNumber - 1];
	int[][] son; son = new int[2 * nodeNumber - 1][2];
	String[] singlebit; singlebit = new String[2 * nodeNumber - 1];
	String[] bitcode; bitcode = new String[2 * nodeNumber - 1];
//  	String[][] huffmanTable; huffmanTable = new String[nodeNumber][2];
	int[][] coordinates; coordinates = new int[2 * nodeNumber - 1][2];

	// Initialize Huffman-Tree
	for (int i = 0; i < nodeNumber; i++) {
	    cumulFrequency[i] = frequency[i];
	    character[i] = alphabet.substring(i, i + 1);
	    dad[i] = -1;
	    son[i][0] = -1;
	    son[i][1] = -1;
	}
	for (int i = nodeNumber; i < cumulFrequency.length; i++) {
	    cumulFrequency[i] = input.length();
	    dad[i] = -1;
	    son[i][0] = -1;
	    son[i][1] = -1;
	}



	/* *****************   Construction Huffman-Tree   ***************** */



	for (int i = 0; i < (nodeNumber - 1); i++) {
	    int min1 = input.length(), min2 = input.length();
	    int index1 = cumulFrequency.length, index2 = cumulFrequency.length;
//	    int pointer = 0;

	    // Choose first sibling, assign dad and the single bit's value
	    for (int j = 0; j < nodeNumber + i; j++) {
		if (cumulFrequency[j] < min1) {
		    min1 = cumulFrequency[j];
		    index1 = j;
		}
	    }
	    cumulFrequency[index1] = input.length();
	    dad[index1] = nodeNumber + i;
	    son[nodeNumber + i][0] = index1;
	    singlebit[index1] = "1";

	    // Choose second sibling, assign dad and the single bit's value
	    for (int j = 0; j < nodeNumber + i; j++) {
		if (cumulFrequency[j] < min2) {
		    min2 = cumulFrequency[j];
		    index2 = j;
		}
	    }
	    cumulFrequency[index2] = input.length();
	    dad[index2] = nodeNumber + i;
	    son[nodeNumber + i][1] = index2;
	    singlebit[index2] = "0";

	    // Create node as dad with cumulated frequency
	    cumulFrequency[nodeNumber + i] = min1 + min2;
	}

	// Set bitcode for root element
	bitcode[2 * nodeNumber - 2] = "";

	// Set bitcodes for internal nodes (as paths to the leaves) and leaves
	for (int i = (2 * nodeNumber - 3); i >= 0; i--) {
	    bitcode[i] = bitcode[dad[i]] + singlebit[i];
	}



	/* ****************   Huffman-Tree - coordinates   **************** */



	// Compute x-coordinates for painting tree
	int startX = 0, stepX = 0;
	if (nodeNumber <= 11) { stepX = 60; }
	else if (nodeNumber <= 13) { stepX = 50; }
	else if (nodeNumber <= 16) { stepX = 40; }
	else if (nodeNumber <= 18) { stepX = 38; }
	startX = 347 - (nodeNumber - 1) * (stepX / 2);

	// Compute y-coordinates for painting tree
	int numberLines = 1, startY = 30, stepY = 0;
	for (int i = 0; i < nodeNumber; i++) {
	    if (bitcode[i].length() > numberLines) {
		numberLines = bitcode[i].length();
	    }
	}
	if (numberLines <= 3) { stepY = 100; }
	else if (numberLines <= 4) { stepY = 80; }
	else if (numberLines <= 6) { stepY = 70; }
	else if (numberLines > 6) { return; }



	/* ****************   Sort (array) data structure   **************** */



	// Sort tree with selection sort
	for (int i = 0; i < nodeNumber - 1; i++) {
	    int selected = i;
	    for (int j = i + 1; j < nodeNumber; j++) {
		for (int k = 0; k < bitcode[j].length(); k++) {
		    // This comparison will find a solution before having
		    // problems with 'EOT' because of being prefix-free
		    if ((int)bitcode[j].charAt(k) <
			(int)bitcode[selected].charAt(k)) {
			selected = j;
			break;
		    }
		    if ((int)bitcode[j].charAt(k) >
			(int)bitcode[selected].charAt(k)) {
			break;
		    }
		}
	    }
	    if (selected > i) {
		int swapI; String swapS;
		swapS = character[selected];
		character[selected] = character[i];
		character[i] = swapS;
		swapI = frequency[selected];
		frequency[selected] = frequency[i];
		frequency[i] = swapI;
		swapI = cumulFrequency[selected];
		cumulFrequency[selected] = cumulFrequency[i];
		cumulFrequency[i] = swapI;
		if (son[dad[selected]][0] == selected) {
		    swapI = son[dad[selected]][0];
		    if (son[dad[i]][0] == i) {
			son[dad[selected]][0] = son[dad[i]][0];
			son[dad[i]][0] = swapI;
		    } else {
			son[dad[selected]][0] = son[dad[i]][1];
			son[dad[i]][1] = swapI;
		    }
		} else {
		    swapI = son[dad[selected]][1];
		    if (son[dad[i]][0] == i) {
			son[dad[selected]][1] = son[dad[i]][0];
			son[dad[i]][0] = swapI;
		    } else {
			son[dad[selected]][1] = son[dad[i]][1];
			son[dad[i]][1] = swapI;
		    }
		}
		swapI = dad[selected];
		dad[selected] = dad[i];
		dad[i] = swapI;
		swapS = singlebit[selected];
		singlebit[selected] = singlebit[i];
		singlebit[i] = swapS;
		swapS = bitcode[selected];
		bitcode[selected] = bitcode[i];
		bitcode[i] = swapS;
	    }
	}



	// Initialize coordinates and reset cumulFrequencies
	for (int i = 0; i < frequency.length; i++) {
	    coordinates[i][0] = startX + i * stepX;
	    coordinates[i][1] = startY + numberLines * stepY;
	    cumulFrequency[i] = frequency[i];
	}
	for (int i = frequency.length; i < cumulFrequency.length; i++) {
	    cumulFrequency[i] = (cumulFrequency[son[i][0]] +
				 cumulFrequency[son[i][1]]);
	    coordinates[i][0] = (coordinates[son[i][0]][0] +
				 coordinates[son[i][1]][0]) / 2;
	    coordinates[i][1] = coordinates[son[i][0]][1] - stepY;
	    if (coordinates[i][1] >= coordinates[son[i][1]][1]) {
		coordinates[i][1] = coordinates[son[i][1]][1] - stepY;
	    }
	}



	/* ******************   Huffman-Tree - output   ****************** */



	// Paint tree's leaves
	for (int i = 0; i < frequency.length; i++) {
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }
	    this.makeLeaf(graf, inCMet, Color.white, character[i],
			  String.valueOf(frequency[i]),
			  coordinates[i][0], coordinates[i][1]);
	    myFrame.repaint();
	}



	// Connect tree's leaves and nodes
	for (int i = frequency.length; i < cumulFrequency.length; i++) {
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	    int xLS = coordinates[son[i][0]][0];
	    int yLS = coordinates[son[i][0]][1];
	    int xLD = coordinates[i][0] - xLS;
	    int yLD = coordinates[i][1] - yLS;

	    // Draw first line
	    for (int j = 1; j < 11; j++) {
		// Delay while drawing line
		try { Thread.sleep(shrtW); } catch (InterruptedException ex) {}
		graf.drawLine(xLS, yLS, xLS + xLD * j/10, yLS + yLD * j/10);
		if (son[i][0] < frequency.length) {
		    this.makeLeaf(graf, inCMet, Color.white,
				  character[son[i][0]],
				  String.valueOf(frequency[son[i][0]]),
				  coordinates[son[i][0]][0],
				  coordinates[son[i][0]][1]);
		} else {
		    this.makeNode(graf, Color.white,
				  String.valueOf(cumulFrequency[son[i][0]]),
				  coordinates[son[i][0]][0],
				  coordinates[son[i][0]][1]);
		}
		myFrame.repaint();
	    }
	    int xRS = coordinates[son[i][1]][0];
	    int yRS = coordinates[son[i][1]][1];
	    int xRD = coordinates[i][0] - xRS;
	    int yRD = coordinates[i][1] - yRS;

	    // Draw second line
	    for (int j = 1; j < 11; j++) {
		// Delay while drawing line
		try { Thread.sleep(shrtW); } catch (InterruptedException ex) {}
		graf.drawLine(xRS, yRS, xRS + xRD * j/10, yRS + yRD * j/10);
		if (son[i][1] < frequency.length) {
		    this.makeLeaf(graf, inCMet, Color.white,
				  character[son[i][1]],
				  String.valueOf(frequency[son[i][1]]),
				  coordinates[son[i][1]][0],
				  coordinates[son[i][1]][1]);
		} else {
		    this.makeNode(graf, Color.white,
				  String.valueOf(cumulFrequency[son[i][1]]),
				  coordinates[son[i][1]][0],
				  coordinates[son[i][1]][1]);
		}
		myFrame.repaint();
	    }

	    // Wait before painting node
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }
	    this.makeNode(graf, Color.white, String.valueOf(cumulFrequency[i]),
			  coordinates[i][0], coordinates[i][1]);
	    myFrame.repaint();

	    // Write bits belonging to the nodes
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }
	    if (yLD < yRD) {
		xRD = xRD * yLD / yRD;
		yRD = yLD;
	    } else if (yRD < yLD) {
		xLD = xLD * yRD / yLD;
		yLD = yRD;
	    }
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }
	    graf.setColor(Color.red);
	    graf.drawString(singlebit[son[i][0]],
			    coordinates[i][0] - xLD / 4 + 10,
			    coordinates[i][1] - yLD / 4);
	    graf.drawString(singlebit[son[i][1]],
			    coordinates[i][0] - xRD / 4 - 20,
			    coordinates[i][1] - yRD / 4);
	    myFrame.repaint();
	}



  	// Create Huffman-Table to code / decode
//  	for (int i = 0; i < nodeNumber; i++) {
//  	    huffmanTable[i][0] = character[i];
//  	    huffmanTable[i][1] = bitcode[i];
//  	}



        // Wait between constructing and coding
        try { Thread.sleep(2 * longW); } catch (InterruptedException ex) { }



	// Call method for coding text
//  	String codedInput = this.code(input, huffmanTable);
//      out.println("\nKodierung von \"" + input + "\": " + codedInput + "\n");



	/* ******************   Encoding-output   ****************** */



	// Code input -> codedString
	String codedString = "", outBitstring = "Code: ";
	graf.setColor(Color.black);
	graf.drawString(outBitstring, 350 - fMet.stringWidth(outBitstring) / 2,
			numberLines * stepY + 80);
	myFrame.repaint();
	// wait after printing "Text"
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	for (int i = 0; i < input.length(); i++) {

	    // Paint leaf
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	    for (int j = 0; j < frequency.length; j++) {
		if(character[j].indexOf(input.charAt(i)) > -1) {
		    int pointer = j;
		    this.makeLeaf(graf, inCMet, Color.orange,
				  character[pointer],
				  String.valueOf(frequency[pointer]),
				  coordinates[pointer][0],
				  coordinates[pointer][1]);
		    myFrame.repaint();

		    // Find path to root
		    do {
			try { Thread.sleep(2 * medW); }
			catch (InterruptedException ex) { }
			graf.setColor(Color.red);
			graf.drawLine(coordinates[pointer][0],
				      coordinates[pointer][1],
				      coordinates[dad[pointer]][0],
				      coordinates[dad[pointer]][1]);
			if (pointer == j) {
			    this.makeLeaf(graf, inCMet, Color.orange,
					  character[pointer],
					  String.valueOf(frequency[pointer]),
					  coordinates[pointer][0],
					  coordinates[pointer][1]);
			    myFrame.repaint();
			} else {
			    this.makeNode(graf, Color.orange, String.
					  valueOf(cumulFrequency[pointer]),
					  coordinates[pointer][0],
					  coordinates[pointer][1]);
			}
			pointer = dad[pointer];
			this.makeNode(graf, Color.orange, String.
				      valueOf(cumulFrequency[pointer]),
				      coordinates[pointer][0],
				      coordinates[pointer][1]);
			myFrame.repaint();
		    } while (dad[pointer] > -1);
		    try { Thread.sleep(2 * medW); }
		    catch (InterruptedException ex) { }
		    graf.setColor(myBackColor);
		    graf.fillRect(340 - fMet.stringWidth(outBitstring) / 2,
				  numberLines * stepY + 65,
				  fMet.stringWidth(outBitstring) + 20, 20);
		    myFrame.repaint();
		    outBitstring += bitcode[j] + " ";
		    codedString += bitcode[j];
		    while (fMet.stringWidth(outBitstring) >= 695) {
			int required = f.getSize() - 1;
			f = new Font("Serif", Font.PLAIN, required);
			fMet = getFontMetrics(f);
			graf.setFont(f);
		    } // to prevent the text from crossing the border...
		    graf.setColor(Color.black);
		    graf.drawString(outBitstring,
				    350 - fMet.stringWidth(outBitstring) / 2,
				    numberLines * stepY + 80);
		    myFrame.repaint();
		    pointer = j;

		    // Wait before resetting tree
		    try { Thread.sleep(2 * medW); }
		    catch (InterruptedException ex) { }
		    for (int k = 0; k < bitcode[j].length(); k++) {
			graf.setColor(Color.black);
			graf.drawLine(coordinates[pointer][0],
				      coordinates[pointer][1],
				      coordinates[dad[pointer]][0],
				      coordinates[dad[pointer]][1]);
			if (pointer == j) {
			    this.makeLeaf(graf, inCMet, Color.white,
					  character[pointer],
					  String.valueOf(frequency[pointer]),
					  coordinates[pointer][0],
					  coordinates[pointer][1]);
			} else {
			    this.makeNode(graf, Color.white, String.
					  valueOf(cumulFrequency[pointer]),
					  coordinates[pointer][0],
					  coordinates[pointer][1]);
			}
			pointer = dad[pointer];
			this.makeNode(graf, Color.white, String.
				      valueOf(cumulFrequency[pointer]),
				      coordinates[pointer][0],
				      coordinates[pointer][1]);
			myFrame.repaint();
		    }
		    break;
		}
	    }
	}



	// Wait between coding and decoding
	try { Thread.sleep(2 * longW); } catch (InterruptedException ex) { }



	// Call method for decoding coded text
//  	String decodedInput = this.decode(codedInput, huffmanTable);
//      out.println("\nDekodierung von \"" + codedInput + "\": " +
//  	            decodedInput + "\n");



	/* ******************   Decoding-output   ****************** */



	// Decode codedString -> decodedBits
	String decodedBits = "", outTextstring = "Text: ";
	int pointer = 2 * nodeNumber - 2;
	graf.setColor(Color.black);
	graf.drawString(outTextstring, 350 - fMet.stringWidth(outTextstring)
			/ 2, numberLines * stepY + 100);
	myFrame.repaint();
	// wait after printing "Text"
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	// Initialize painting tree
	this.makeNode(graf, Color.orange,
		      String.valueOf(cumulFrequency[pointer]),
		      coordinates[pointer][0], coordinates[pointer][1]);
	myFrame.repaint();
	//Wait after first (root) node
	try { Thread.sleep(2 * medW); } catch (InterruptedException ex) { }

	for (int i = 0; i < codedString.length(); i++) {
	    // Choose son belonging to current bit
	    if (String.valueOf(codedString.charAt(i)).
		compareTo(singlebit[son[pointer][0]]) == 0) {
		graf.setColor(Color.red);
		graf.drawLine(coordinates[pointer][0],
			      coordinates[pointer][1],
			      coordinates[son[pointer][0]][0],
			      coordinates[son[pointer][0]][1]);
		this.makeNode(graf, Color.orange, String.
			      valueOf(cumulFrequency[pointer]),
			      coordinates[pointer][0],
			      coordinates[pointer][1]);
		pointer = son[pointer][0];
	    } else {
		graf.setColor(Color.red);
		graf.drawLine(coordinates[pointer][0],
			      coordinates[pointer][1],
			      coordinates[son[pointer][1]][0],
			      coordinates[son[pointer][1]][1]);
		this.makeNode(graf, Color.orange, String.
			      valueOf(cumulFrequency[pointer]),
			      coordinates[pointer][0],
			      coordinates[pointer][1]);
		pointer = son[pointer][1];
	    }

	    // If leaf is found reset tree
	    if (son[pointer][0] == -1) {
		this.makeLeaf(graf, inCMet, Color.orange, character[pointer],
			      String.valueOf(frequency[pointer]),
			      coordinates[pointer][0],
			      coordinates[pointer][1]);
		myFrame.repaint();
		try { Thread.sleep(2 * medW); }
		catch (InterruptedException ex) { }
		graf.setColor(myBackColor);
		graf.fillRect(340 - fMet.stringWidth(outTextstring) / 2,
			      numberLines * stepY + 85,
			      fMet.stringWidth(outTextstring) + 20, 20);
		myFrame.repaint();
		outTextstring+= character[pointer] + " ";
		decodedBits += character[pointer];
		graf.setColor(Color.black);
		graf.drawString(outTextstring,
				350 - fMet.stringWidth(outTextstring) / 2,
				numberLines * stepY + 100);
		myFrame.repaint();
		try { Thread.sleep(2 * medW); }
		catch (InterruptedException ex) { }
		do {
		    graf.setColor(Color.black);
		    graf.drawLine(coordinates[pointer][0],
				  coordinates[pointer][1],
				  coordinates[dad[pointer]][0],
				  coordinates[dad[pointer]][1]);
		    if (son[pointer][0] == -1) {
			this.makeLeaf(graf, inCMet, Color.white,
				      character[pointer],
				      String.valueOf(frequency[pointer]),
				      coordinates[pointer][0],
				      coordinates[pointer][1]);
		    } else {
			this.makeNode(graf, Color.white, String.
				      valueOf(cumulFrequency[pointer]),
				      coordinates[pointer][0],
				      coordinates[pointer][1]);
		    }
		    pointer = dad[pointer];
		    this.makeNode(graf, Color.white, String.
				  valueOf(cumulFrequency[pointer]),
				  coordinates[pointer][0],
				  coordinates[pointer][1]);
		    myFrame.repaint();
		} while (dad[pointer] != -1);
		myFrame.repaint();
		try { Thread.sleep(2 * medW); }
		catch (InterruptedException ex) { }
		this.makeNode(graf, Color.white, String.
			      valueOf(cumulFrequency[pointer]),
			      coordinates[pointer][0],
			      coordinates[pointer][1]);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}
		if (i < (codedString.length() - 1)) {
		    this.makeNode(graf, Color.orange, String.
			       valueOf(cumulFrequency[pointer]),
			       coordinates[pointer][0],
			       coordinates[pointer][1]);
		    myFrame.repaint();
		}
	    } else {
		this.makeNode(graf, Color.orange, String.
			      valueOf(cumulFrequency[pointer]),
			      coordinates[pointer][0],
			      coordinates[pointer][1]);
	    }
	    myFrame.repaint();
	    try { Thread.sleep(2 * medW); } catch (InterruptedException ex) { }
	}
    }



    /* ******************   Former methods for textmode   ****************** */



//  public String code(String codableText, String[][] huffmanTable) {
//  	String returnCode = ""; // coded text (without spaces) to return
//  	String bitstring = ""; // coded text with spaces for human eyes reading

//  	for (int i = 0; i < codableText.length(); i++) {
//  	    for (int j = 0; j < huffmanTable.length; j++) {
//  		if(huffmanTable[j][0].indexOf(codableText.charAt(i)) > -1) {
//  		    returnCode += huffmanTable[j][1];
//  		    bitstring += huffmanTable[j][1] + " ";
//  		    out.println("Zeichen: " + huffmanTable[j][0] + " -> " +
//  				huffmanTable[j][1]);
//  		    break;
//  		}
//  	    }
//  	}
//  	bitstring = bitstring.substring(0, bitstring.length() - 1);
//  	out.println("Zeichenfolge \"" + codableText + "\" ergibt Bitfolge \"" +
//  		    bitstring + "\".");
//  	return returnCode;
//  }

//  public String decode(String textableCode, String[][] huffmanTable) {
//  	String returnText = ""; // decoded code to return
//  	textableCode += " ";
//  	String comparestring; // bitstring to look for in Huffman-Table
//  	int start = 0;

//  	for (int i = 1; i < textableCode.length(); i++) {
//   	    comparestring = textableCode.substring(start, i);
//  	    out.print("Bitfolge: " + comparestring);
//  	    for (int j = 0; j < huffmanTable.length; j++) {
//  		if (huffmanTable[j][1].compareTo(comparestring) == 0) {
//  		    returnText += huffmanTable[j][0];
//  		    start = i;
//  		    out.print("\t = " + huffmanTable[j][0]);
//  		    break;
//  		}
//  	    }
//  	    out.println();
//  	}
//  	return returnText;
//  }

//  public static void main(String[] args) {
//  	Huffplet h = new Huffplet();

//  	if (args.length == 0) {
//  	    h.simulate("ROKOKOKOKOTTEN");
//  	} else {
//  	    h.simulate(args[0].toUpperCase());
//  	}
//  }

}

